@@ -22,10 +22,14 @@
 #include "cttask.h"
 #include "ctvariable.h"
 
+// For root permissions
+#include <KAuthAction>
+#include <KAuthExecuteJob>
+
 #include <pwd.h> // pwd, getpwnam(), getpwuid()
 #include <unistd.h> // getuid(), unlink()
 
-#include "logging.h"
+#include "kcm_cron_debug.h"
 
 CommandLineStatus CommandLine::execute()
 {
@@ -97,7 +101,7 @@ CTCron::CTCron(const QString &crontabBinary, const struct passwd *userInfos, boo
 
     if (!initializeFromUserInfos(userInfos)) {
         ctInitializationError.setErrorMessage(i18n("No password entry found for uid '%1'", getuid()));
-        logDebug() << "Error in crontab creation of" << userInfos->pw_name;
+        qCDebug(KCM_CRON_LOG) << "Error in crontab creation of" << userInfos->pw_name;
         return;
     }
 
@@ -106,9 +110,9 @@ CTCron::CTCron(const QString &crontabBinary, const struct passwd *userInfos, boo
     if (commandLineStatus.exitCode == 0) {
         this->parseFile(d->tmpFileName);
     } else {
-        logDebug() << "Error when executing command" << commandLineStatus.commandLine;
-        logDebug() << "Standard output :" << commandLineStatus.standardOutput;
-        logDebug() << "Standard error :" << commandLineStatus.standardError;
+        qCDebug(KCM_CRON_LOG) << "Error when executing command" << commandLineStatus.commandLine;
+        qCDebug(KCM_CRON_LOG) << "Standard output :" << commandLineStatus.standardOutput;
+        qCDebug(KCM_CRON_LOG) << "Standard error :" << commandLineStatus.standardError;
     }
 
     d->initialTaskCount = d->task.size();
@@ -118,6 +122,7 @@ CTCron::CTCron(const QString &crontabBinary, const struct passwd *userInfos, boo
 CTCron::CTCron()
     : d(new CTCronPrivate())
 {
+    // Empty constructor, new CTCronPrivate created.
 }
 
 bool CTCron::initializeFromUserInfos(const struct passwd *userInfos)
@@ -137,8 +142,9 @@ CTCron &CTCron::operator=(const CTCron &source)
         return *this;
     }
 
+    // Not sure when this would tigger.
     if (source.isSystemCron()) {
-        logDebug() << "Affect the system cron";
+        qCDebug(KCM_CRON_LOG) << "Affect the system cron";
     }
 
     d->variable.clear();
@@ -225,12 +231,12 @@ QString CTCron::exportCron() const
 {
     QString exportCron;
 
-    for (CTVariable *ctVariable : qAsConst(d->variable)) {
+    for (CTVariable *ctVariable : std::as_const(d->variable)) {
         exportCron += ctVariable->exportVariable();
         exportCron += QLatin1String("\n");
     }
 
-    for (CTTask *ctTask : qAsConst(d->task)) {
+    for (CTTask *ctTask : std::as_const(d->task)) {
         exportCron += ctTask->exportTask();
         exportCron += QLatin1String("\n");
     }
@@ -258,7 +264,7 @@ bool CTCron::saveToFile(const QString &fileName)
         return false;
     }
 
-    // logDebug() << exportCron();
+    // qCDebug(KCM_CRON_LOG) << exportCron();
 
     QTextStream out(&file);
     out << exportCron();
@@ -306,28 +312,46 @@ CTSaveStatus CTCron::save()
         return CTSaveStatus(i18n("Unable to open crontab file for writing"), i18n("The file %1 could not be opened.", d->tmpFileName));
     }
 
-    CommandLineStatus commandLineStatus = d->writeCommandLine.execute();
-    // install temp file into crontab
-    if (commandLineStatus.exitCode != 0) {
+    // For root permissions.
+    if (d->systemCron) {
+        qCDebug(KCM_CRON_LOG) << "Attempting to save system cron";
+        QVariantMap args;
+        args.insert(QStringLiteral("source"), d->tmpFileName);
+        args.insert(QStringLiteral("target"), d->writeCommandLine.standardOutputFile);
+        KAuth::Action saveAction(QStringLiteral("local.kcron.crontab.save"));
+        saveAction.setHelperId(QStringLiteral("local.kcron.crontab"));
+        saveAction.setArguments(args);
+        KAuth::ExecuteJob *job = saveAction.execute();
+        if (!job->exec())
+            qCDebug(KCM_CRON_LOG) << "KAuth returned an error: " << job->error() << job->errorText();
         QFile::remove(d->tmpFileName);
-        return prepareSaveStatusError(commandLineStatus);
-    } else {
-        // Remove the temp file
+        if (job->error() > 0) {
+            return CTSaveStatus(i18n("KAuth::ExecuteJob Error"), job->errorText());
+        }
+    }
+    // End root permissions.
+    else {
+        qCDebug(KCM_CRON_LOG) << "Attempting to save user cron";
+        // Save without root permissions.
+        const CommandLineStatus commandLineStatus = d->writeCommandLine.execute();
         QFile::remove(d->tmpFileName);
+        if (commandLineStatus.exitCode != 0) {
+            return prepareSaveStatusError(commandLineStatus);
+        }
     }
 
     // Mark as applied
-    for (CTTask *ctTask : qAsConst(d->task)) {
+    for (CTTask *ctTask : std::as_const(d->task)) {
         ctTask->apply();
     }
 
-    for (CTVariable *ctVariable : qAsConst(d->variable)) {
+    for (CTVariable *ctVariable : std::as_const(d->variable)) {
         ctVariable->apply();
     }
 
     d->initialTaskCount = d->task.size();
     d->initialVariableCount = d->variable.size();
-
+    qCDebug(KCM_CRON_LOG) << "All saved";
     return CTSaveStatus();
 }
 
@@ -354,13 +378,13 @@ bool CTCron::isDirty() const
         return true;
     }
 
-    for (CTTask *ctTask : qAsConst(d->task)) {
+    for (CTTask *ctTask : std::as_const(d->task)) {
         if (ctTask->dirty()) {
             return true;
         }
     }
 
-    for (CTVariable *ctVariable : qAsConst(d->variable)) {
+    for (CTVariable *ctVariable : std::as_const(d->variable)) {
         if (ctVariable->dirty()) {
             return true;
         }
@@ -373,7 +397,7 @@ QString CTCron::path() const
 {
     QString path;
 
-    for (CTVariable *ctVariable : qAsConst(d->variable)) {
+    for (CTVariable *ctVariable : std::as_const(d->variable)) {
         if (ctVariable->variable == QLatin1String("PATH")) {
             path = ctVariable->value;
         }
@@ -401,7 +425,7 @@ void CTCron::addTask(CTTask *task)
         task->setSystemCrontab(false);
     }
 
-    logDebug() << "Adding task" << task->comment << " user : " << task->userLogin;
+    qCDebug(KCM_CRON_LOG) << "Adding task" << task->comment << " user : " << task->userLogin;
 
     d->task.append(task);
 }
@@ -414,7 +438,7 @@ void CTCron::addVariable(CTVariable *variable)
         variable->userLogin = d->userLogin;
     }
 
-    logDebug() << "Adding variable" << variable->variable << " user : " << variable->userLogin;
+    qCDebug(KCM_CRON_LOG) << "Adding variable" << variable->variable << " user : " << variable->userLogin;
 
     d->variable.append(variable);
 }
